#! /usr/bin/env python
# -*- coding: utf-8 -*-

import rospy
import actionlib
import math
import copy
import moveit_commander
import moveit_msgs.msg
from geometry_msgs.msg import Quaternion, Pose, PoseStamped, Vector3
from seed_r7_ros_controller.srv import*
from tf.transformations import quaternion_from_euler
from move_base_msgs.msg import MoveBaseAction, MoveBaseGoal

def euler_to_quaternion(role, pitch, yaw):
    q = quaternion_from_euler(role, pitch, yaw)
    return Quaternion(q[0], q[1], q[2], q[3])

def goal_pose(pose):
    goal_pose = MoveBaseGoal()
    goal_pose.target_pose.header.frame_id = 'map'
    goal_pose.target_pose.pose.position.x = pose[0][0]
    goal_pose.target_pose.pose.position.y = pose[0][1]
    goal_pose.target_pose.pose.position.z = pose[0][2]
    goal_pose.target_pose.pose.orientation.x = pose[1][0]
    goal_pose.target_pose.pose.orientation.y = pose[1][1]
    goal_pose.target_pose.pose.orientation.z = pose[1][2]
    goal_pose.target_pose.pose.orientation.w = pose[1][3]
    return goal_pose

######################################
# MAIN（お客さん（子供）に商品棚の商品を渡して、手を振る）
######################################

def main():

    # init node
    rospy.init_node("moveit_tutorial_node")

    # configuration for moveit
    robot = moveit_commander.RobotCommander()

    # hand control service
    rospy.wait_for_service('/seed_r7_ros_controller/hand_control')
    service = rospy.ServiceProxy('/seed_r7_ros_controller/hand_control', HandControl)

    # group name
    rarm = moveit_commander.MoveGroupCommander("rarm")
    larm = moveit_commander.MoveGroupCommander("larm")
    lifter = moveit_commander.MoveGroupCommander("lifter")
    upper = moveit_commander.MoveGroupCommander("upper_body")
    waist = moveit_commander.MoveGroupCommander("waist")
    rarm_waist = moveit_commander.MoveGroupCommander("rarm_with_waist")
    larm_waist = moveit_commander.MoveGroupCommander("larm_with_waist")
    rarm_waist.set_end_effector_link("r_eef_pick_link")
    larm_waist.set_end_effector_link("l_eef_pick_link")

    # display of joint name
    print ("")
    print ("RARM Joint names:", robot.get_joint_names("rarm"))
    print ("")
    print ("LARM Joint names:", robot.get_joint_names("larm"))
    print ("")
    print ("RARM_WITH_WAIST Joint names:", robot.get_joint_names("rarm_with_waist"))
    print ("")
    print ("LARM_WITH_WAIST Joint names:", robot.get_joint_names("larm_with_waist"))
    print ("")
    print ("UPPER_BODY Joint names:", robot.get_joint_names("upper_body"))
    print ("")
    print ("LIFTER Joint names", robot.get_joint_names("lifter"))
    print ("")
    print ("WAIST Joint names", robot.get_joint_names("waist"))

    # release
    print ("")
    print ("reset hand (release)")
    response = service(0, 'release', 100)  #Right Hand
    response = service(1, 'release', 100)  #Left Hand
    rospy.sleep(3)

    rospy.init_node('test_nav')

    client = actionlib.SimpleActionClient('move_base', MoveBaseAction)
    client.wait_for_server()

    for pose in waypoints:
        goal = goal_pose(pose)
        client.send_goal(goal)
        client.wait_for_result()

    # reset pose
    print ("")
    print ("reset pose")
    joint_goal = upper.get_current_joint_values()
    for i in range(0,len(joint_goal)):
        joint_goal[i] = 0
    joint_goal[6] = -3
    joint_goal[16] = -3
    upper.go(joint_goal, wait=True) 

    # lifter down
    lifter_goal = lifter.get_current_joint_values()
    for i in range(0, len(lifter_goal)):
        lifter_goal[i] = 0
    lifter_goal[0] = 0.9191
    lifter_goal[1] = -0.9191
    lifter_goal[2] = -0.9191
    lifter_goal[3] = 0.9191
    lifter.go(lifter_goal, wait=True)

    # trun left
    joint_goal = upper.get_current_joint_values()
    for i in range(0,len(joint_goal)):
        joint_goal[i] = 0
    joint_goal[0] = 1.57
    joint_goal[6] = -3
    joint_goal[16] = -3
    upper.go(joint_goal, wait=True) 

    rospy.sleep(1.5)

    ######################################
    # pick a merchandise（商品棚から商品を取る）
    ######################################

    print ("")
    print ("Pose 1: approach")
    pose_goal1 = Pose()
    pose_goal1.orientation.x = 0.334
    pose_goal1.orientation.y = 0.617
    pose_goal1.orientation.z = 0.365
    pose_goal1.orientation.w = 0.611
    pose_goal1.position.x = -0.210
    pose_goal1.position.y = 0.620
    pose_goal1.position.z = 0.957
    larm_waist.set_pose_target(pose_goal1)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(1.5)

    print ("")
    print ("Pose 2: hand down")
    pose_goal2 = Pose()
    pose_goal2.orientation.x = 0.216
    pose_goal2.orientation.y = 0.668
    pose_goal2.orientation.z = 0.246
    pose_goal2.orientation.w = 0.668
    pose_goal2.position.x = -0.210
    pose_goal2.position.y = 0.620
    pose_goal2.position.z = 0.870
    larm_waist.set_pose_target(pose_goal2)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("left hand grasp")
    response = service(1, 'grasp', 100)
    rospy.sleep(5)

    print ("")
    print ("pose 3: hand up")
    pose_goal3 = Pose()
    pose_goal3.orientation.x = 0.412
    pose_goal3.orientation.y = 0.565
    pose_goal3.orientation.z = 0.415
    pose_goal3.orientation.w = 0.582
    pose_goal3.position.x = -0.198
    pose_goal3.position.y = 0.558
    pose_goal3.position.z = 1.063
    larm_waist.set_pose_target(pose_goal3)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("Pose 4: turn right")
    pose_goal4 = Pose()
    pose_goal4.orientation.x = 0.730
    pose_goal4.orientation.y = -0.159
    pose_goal4.orientation.z = 0.108
    pose_goal4.orientation.w = 0.656
    pose_goal4.position.x = 0.442
    pose_goal4.position.y = 0.202
    pose_goal4.position.z = 1.174
    larm_waist.set_pose_target(pose_goal4)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(3)

    ######################################
    # take a merchandise（商品を渡す）
    ######################################

    print ("")
    print ("Pose 5: approach")
    pose_goal5 = Pose()
    pose_goal5.orientation.x = 0.730
    pose_goal5.orientation.y = -0.159
    pose_goal5.orientation.z = -0.108
    pose_goal5.orientation.w = 0.656
    pose_goal5.position.x = 0.400
    pose_goal5.position.y = 0.202
    pose_goal5.position.z = 1.174
    larm.set_pose_target(pose_goal5)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(3)

    print ("")
    print ("left hand release")
    response = service(1, 'release', 100)

    rospy.sleep(3)

    print ("")
    print ("reset pose")
    joint_goal = upper.get_current_joint_values()
    for i in range(0, len(joint_goal)):
        joint_goal[i] = 0
    joint_goal[6] = -3
    joint_goal[16] = -3
    upper.go(joint_goal, wait=True) 

    rospy.sleep(3)

    ######################################
    # wave a hand（手を振る）
    ######################################

    print ("")
    print ("Pose 6: hand up")
    pose_goal6 = Pose()
    pose_goal6.orientation.x = 0.730
    pose_goal6.orientation.y = 0.159
    pose_goal6.orientation.z = -0.175
    pose_goal6.orientation.w = 0.641
    pose_goal6.position.x = 0.400
    pose_goal6.position.y = 0.172
    pose_goal6.position.z = 1.011
    larm.set_pose_target(pose_goal6)
    print ("LARM_WITH_WAIST", larm_waist.go(wait=True))

    rospy.sleep(3)

    print ("")
    print ("Pose 7: wave hand left")
    pose_goal7 = Pose()
    pose_goal7.orientation.x = 0.666
    pose_goal7.orientation.y = -0.170
    pose_goal7.orientation.z = 0.093
    pose_goal7.orientation.w = 0.720
    pose_goal7.position.x = 0.415
    pose_goal7.position.y = 0.350
    pose_goal7.position.z = 1.338
    larm.set_pose_target(pose_goal7)
    print ("LARM", larm.go(wait=True))

    rospy.sleep(3)

    print ("")
    print ("Pose 8: wave hand right")
    pose_goal8 = Pose()
    pose_goal8.orientation.x = 0.666
    pose_goal8.orientation.y = -0.170
    pose_goal8.orientation.z = 0.093
    pose_goal8.orientation.w = 0.720
    pose_goal8.position.x = 0.415
    pose_goal8.position.y = 0.330
    pose_goal8.position.z = 1.338
    larm.set_pose_target(pose_goal8)
    print ("LARM", larm.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("Pose 7: wave hand left")
    pose_goal7 = Pose()
    pose_goal7.orientation.x = 0.666
    pose_goal7.orientation.y = -0.170
    pose_goal7.orientation.z = 0.093
    pose_goal7.orientation.w = 0.720
    pose_goal7.position.x = 0.415
    pose_goal7.position.y = 0.350
    pose_goal7.position.z = 1.338
    larm.set_pose_target(pose_goal7)
    print ("LARM", larm.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("Pose 8: wave hand right")
    pose_goal8 = Pose()
    pose_goal8.orientation.x = 0.666
    pose_goal8.orientation.y = -0.170
    pose_goal8.orientation.z = 0.093
    pose_goal8.orientation.w = 0.720
    pose_goal8.position.x = 0.415
    pose_goal8.position.y = 0.330
    pose_goal8.position.z = 1.338
    larm.set_pose_target(pose_goal8)
    print ("LARM", larm.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("Pose 7: wave hand left")
    pose_goal7 = Pose()
    pose_goal7.orientation.x = 0.666
    pose_goal7.orientation.y = -0.170
    pose_goal7.orientation.z = 0.093
    pose_goal7.orientation.w = 0.720
    pose_goal7.position.x = 0.415
    pose_goal7.position.y = 0.350
    pose_goal7.position.z = 1.338
    larm.set_pose_target(pose_goal7)
    print ("LARM", larm.go(wait=True))

    rospy.sleep(1)

    print ("")
    print ("reset pose")
    joint_goal = upper.get_current_joint_values()
    for i in range(0, len(joint_goal)):
        joint_goal[i] = 0
    joint_goal[6] = -3
    joint_goal[16] = -3
    upper.go(joint_goal, wait=True) 

    lifter_goal = lifter.get_current_joint_values()
    for i in range(0, len(lifter_goal)):
        lifter_goal[i] = 0
    lifter.go(lifter_goal, wait=True)  

    print ("")
    print ("task finished!")

if __name__ == '__main__':
    try:
        main()
    except rospy.ROSInterruptException:
        pass